C$Procedure     MKPLAT  ( Create a plate model kernel )
 
      PROGRAM   MKPLAT
 
      IMPLICIT  NONE
 
C$ Abstract
C
C     Utility program to create a triangular plate model kernel.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required Reading
C
C     MKPLAT User's Guide
C     EK
C     SURFACE
C
C$ Author_and_Institution
C
C     J.A. Bytof     (JPL)
C
C$ Version
C
C-    Version 2.0.2, 23-Mar-1999 (JAB)
C
C        Implemented a first-order elevation mask.
C
C-    Version 2.0.1, 30-Jan-1999 (JAB)
C
C        Minor changes.
C
C-    Version 2.0.0, 06-Dec-1998 (JAB)
C
C        Now has voxelization capability.
C
C        User defined comments read from a text file are stored.
C
C        Added segments containing plate area, unit
C        normals, plate occlusion flags, length of
C        longest side.
C
C        Added a preliminary horizon mask array.
C
C
C-    Version 1.0.1, 02-May-1997 (JAB)
C
C        The user can now enter a made-up body,
C        as long as its name nor its NAIF ID matches a
C        preexisting one.
C
C-    Version 1.0.0, 09-APR-1997 (JAB)
C
C-&
 
C
C     SPICELIB functions
C
 
      LOGICAL                EXISTS
      INTEGER                LASTNB
 
C
C     Global parameters
C
 
      INCLUDE                'pltmax.inc'
      INCLUDE                'ektype.inc'
      INCLUDE                'ekcnamsz.inc'
      INCLUDE                'ektnamsz.inc'
 
 
C
C     The parameters here are for ease in maintaining the
C     large collection of database segment/column data that
C     are stored in data statements.  To insert a column in the
C     block from BEGx to ENDx, redefine Nxx to be
C     one larger than its current definition.  Recompiling
C     will automatically modify all the other parameters.
C
 
      INTEGER               N01
      PARAMETER           ( N01 = 9 )
      INTEGER               BEG01
      PARAMETER           ( BEG01 = 1 )
      INTEGER               END01
      PARAMETER           ( END01 = BEG01 + N01 - 1 )
 
      INTEGER               N02
      PARAMETER           ( N02 = 4 )
      INTEGER               BEG02
      PARAMETER           ( BEG02 = END01 + 1 )
      INTEGER               END02
      PARAMETER           ( END02 = BEG02 + N02 - 1 )
 
      INTEGER               N03
      PARAMETER           ( N03 = 4 )
      INTEGER               BEG03
      PARAMETER           ( BEG03 = END02 + 1 )
      INTEGER               END03
      PARAMETER           ( END03 = BEG03 + N03 - 1 )
 
      INTEGER               N04
      PARAMETER           ( N04 = 1 )
      INTEGER               BEG04
      PARAMETER           ( BEG04 = END03 + 1 )
      INTEGER               END04
      PARAMETER           ( END04 = BEG04 + N04 - 1 )
 
      INTEGER               N05
      PARAMETER           ( N05 = 8 )
      INTEGER               BEG05
      PARAMETER           ( BEG05 = END04 + 1 )
      INTEGER               END05
      PARAMETER           ( END05 = BEG05 + N05 - 1 )
 
      INTEGER               N06
      PARAMETER           ( N06 = 9 )
      INTEGER               BEG06
      PARAMETER           ( BEG06 = END05 + 1 )
      INTEGER               END06
      PARAMETER           ( END06 = BEG06 + N06 - 1 )
 
      INTEGER               N07
      PARAMETER           ( N07 = 1 )
      INTEGER               BEG07
      PARAMETER           ( BEG07 = END06 + 1 )
      INTEGER               END07
      PARAMETER           ( END07 = BEG07 + N07 - 1 )
 
      INTEGER               N08
      PARAMETER           ( N08 = 1 )
      INTEGER               BEG08
      PARAMETER           ( BEG08 = END07 + 1 )
      INTEGER               END08
      PARAMETER           ( END08 = BEG08 + N08 - 1 )
 
      INTEGER               N09
      PARAMETER           ( N09 = 2 )
      INTEGER               BEG09
      PARAMETER           ( BEG09 = END08 + 1 )
      INTEGER               END09
      PARAMETER           ( END09 = BEG09 + N09 - 1 )
 
      INTEGER               N10
      PARAMETER           ( N10 = 1 )
      INTEGER               BEG10
      PARAMETER           ( BEG10 = END09 + 1 )
      INTEGER               END10
      PARAMETER           ( END10 = BEG10 + N10 - 1 )
 
      INTEGER               N11
      PARAMETER           ( N11 = 1 )
      INTEGER               BEG11
      PARAMETER           ( BEG11 = END10 + 1 )
      INTEGER               END11
      PARAMETER           ( END11 = BEG11 + N11 - 1 )
 
      INTEGER               N12
      PARAMETER           ( N12 = 1 )
      INTEGER               BEG12
      PARAMETER           ( BEG12 = END11 + 1 )
      INTEGER               END12
      PARAMETER           ( END12 = BEG12 + N12 - 1 )
 
      INTEGER               N13
      PARAMETER           ( N13 = 1 )
      INTEGER               BEG13
      PARAMETER           ( BEG13 = END12 + 1 )
      INTEGER               END13
      PARAMETER           ( END13 = BEG13 + N13 - 1 )
 
      INTEGER               N14
      PARAMETER           ( N14 = 6 )
      INTEGER               BEG14
      PARAMETER           ( BEG14 = END13 + 1 )
      INTEGER               END14
      PARAMETER           ( END14 = BEG14 + N14 - 1 )
 
 
      INTEGER               NCOLS
      PARAMETER           ( NCOLS = END14     )
 
 
 
C
C     Local variables
C
      INTEGER                I
      INTEGER                N
      INTEGER                IC
      INTEGER                LB
      INTEGER                NVC
      INTEGER                NVX
      INTEGER                NREC
      INTEGER                NCOL
      INTEGER                NLINE
      INTEGER                SEGNO
      INTEGER                BCODE
      INTEGER                HANDLE
      INTEGER                STDOUT
      INTEGER                NVXTOT
      INTEGER                NP       ( 1 )
      INTEGER                NV       ( 1 )
      INTEGER                VERSN    ( 1 )
      INTEGER                NAIFID   ( 1 )
      INTEGER                ENTSZS   ( 1 )
      INTEGER                NVOX     ( 3 )
      INTEGER                DTYPES   ( NCOLS  )
      INTEGER                PMASK    ( MAXPLT )
      INTEGER                V1       ( MAXPLT )
      INTEGER                V2       ( MAXPLT )
      INTEGER                V3       ( MAXPLT )
      INTEGER                VID      ( MAXVRT )
      INTEGER                PID      ( MAXPLT )
      INTEGER                WKINDX   ( MAXNPV )
      INTEGER                RCPTRS   ( MAXNPV )
      INTEGER                VXPNTR   ( MAXNPV )
      INTEGER                VXLIST   ( MAXNPV )
      INTEGER                VCPNTR   ( MAXNPV )
      INTEGER                VCLIST   ( MAXNPV )
 
 
      CHARACTER*(80)         LINE
      CHARACTER*(18)         BVID
      CHARACTER*(24)         DATE
      CHARACTER*(80)         BNAME
      CHARACTER*(80)         CMDFIL
      CHARACTER*(TNAMSZ)     TABNAM
      CHARACTER*(200)        DECLAR
      CHARACTER*(255)        INCOM
      CHARACTER*(16)         DT       ( 4 )
      CHARACTER*(24)         TIM      ( 1 )
      CHARACTER*(80)         CREATR   ( 1 )
      CHARACTER*(80)         BDNAME   ( 1 )
      CHARACTER*(80)         INFILE   ( 1 )
      CHARACTER*(80)         OUTFIL   ( 1 )
      CHARACTER*(80)         MODESC   ( 1 )
      CHARACTER*(80)         ALBFIL   ( 1 )
      CHARACTER*(80)         COMFIL   ( 1 )
      CHARACTER*(80)         LSFILE   ( 1 )
      CHARACTER*( 1)         HORFLG   ( 1 )
      CHARACTER*( 2)         SIZE     ( NCOLS  )
      CHARACTER*(CNAMSZ)     CNAMES   ( NCOLS  )
      CHARACTER*(10)         IDXFLG   ( NCOLS  )
      CHARACTER*(10)         FXDCNT   ( NCOLS  )
      CHARACTER*(10)         NULLOK   ( NCOLS  )
      CHARACTER*(200)        CDECLS   ( NCOLS  )
      CHARACTER*(MAXCLN)     COMMNT   ( MAXCOM )
 
 
      DOUBLE PRECISION       XMIN
      DOUBLE PRECISION       XMAX
      DOUBLE PRECISION       YMIN
      DOUBLE PRECISION       YMAX
      DOUBLE PRECISION       ZMIN
      DOUBLE PRECISION       ZMAX
      DOUBLE PRECISION       VOXSIZ
      DOUBLE PRECISION       VOXSCL
      DOUBLE PRECISION       VOXORI   ( 3 )
      DOUBLE PRECISION       EXTENT   ( 6 )
      DOUBLE PRECISION       X        ( MAXVRT )
      DOUBLE PRECISION       Y        ( MAXVRT )
      DOUBLE PRECISION       Z        ( MAXVRT )
      DOUBLE PRECISION       AREA     ( MAXPLT )
      DOUBLE PRECISION       LSIDE    ( MAXPLT )
      DOUBLE PRECISION       NORMX    ( MAXPLT )
      DOUBLE PRECISION       NORMY    ( MAXPLT )
      DOUBLE PRECISION       NORMZ    ( MAXPLT )
      DOUBLE PRECISION       CENTX    ( MAXPLT )
      DOUBLE PRECISION       CENTY    ( MAXPLT )
      DOUBLE PRECISION       CENTZ    ( MAXPLT )
      DOUBLE PRECISION       ALBEDO   ( MAXPLT )
      DOUBLE PRECISION       MAXELE   ( MAXPLT )
      DOUBLE PRECISION       CENT     ( 3, MAXPLT )
      DOUBLE PRECISION       NORM     ( 3, MAXPLT )
      DOUBLE PRECISION       VERT     ( 3, 3, MAXPLT )
 
 
      LOGICAL                EOF
      LOGICAL                FOUND
      LOGICAL                BODFND
      LOGICAL                CODFND
      LOGICAL                NLFLGS   ( 1 )
 
 
 
      SAVE
 
 
 
 
      DATA  DT     / 'CHARACTER*(*)',
     .               'DOUBLE PRECISION',
     .               'INTEGER',
     .               'TIME'             /
 
 
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG01, END01 )
     .    /
     .       'MODEL_DESCRIPTION', '1', CHR,  'TRUE', 'FALSE', 'FALSE',
     .       'CREATED',           '1', CHR,  'TRUE', 'FALSE', 'FALSE',
     .       'CREATOR',           '1', CHR,  'TRUE', 'FALSE', 'FALSE',
     .       'BODY_NAME',         '1', CHR,  'TRUE', 'FALSE', 'FALSE',
     .       'NAIF_ID',           '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'VERSION',           '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'NUM_VERTICES',      '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'NUM_PLATES',        '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'NLINES',            '1', INT,  'TRUE', 'FALSE', 'FALSE' /
 
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG02, END02 )
     .    /
     .       'VERTEX_ID',         '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'VERT_X',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'VERT_Y',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'VERT_Z',            '1',  DP,  'TRUE', 'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG03, END03 )
     .    /
     .       'PLATE_ID',          '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'VERT_1',            '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'VERT_2',            '1', INT,  'TRUE', 'FALSE', 'FALSE',
     .       'VERT_3',            '1', INT,  'TRUE', 'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG04, END04 )
     .    /
     .       'ALBEDO',            '1',  DP,  'TRUE', 'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG05, END05 )
     .    /
     .       'NORM_X',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'NORM_Y',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'NORM_Z',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'CENT_X',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'CENT_Y',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'CENT_Z',            '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'AREA',              '1',  DP,  'TRUE', 'FALSE', 'FALSE',
     .       'LONG_SIDE',         '1',  DP,  'TRUE', 'FALSE', 'FALSE' /
 
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG06, END06 )
     .    /
     .      'NVOX_X',             '1', INT,  'FALSE', 'FALSE', 'FALSE',
     .      'NVOX_Y',             '1', INT,  'FALSE', 'FALSE', 'FALSE',
     .      'NVOX_Z',             '1', INT,  'FALSE', 'FALSE', 'FALSE',
     .      'VOXSIZ',             '1',  DP,  'FALSE', 'FALSE', 'FALSE',
     .      'VOXORI_X',           '1',  DP,  'FALSE', 'FALSE', 'FALSE',
     .      'VOXORI_Y',           '1',  DP,  'FALSE', 'FALSE', 'FALSE',
     .      'VOXORI_Z',           '1',  DP,  'FALSE', 'FALSE', 'FALSE',
     .      'NVXTOT',             '1', INT,  'FALSE', 'FALSE', 'FALSE',
     .      'NVX',                '1', INT,  'FALSE', 'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG07, END07 )
     .    /
     .      'VXPNTR',             '1', INT,  'TRUE',  'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG08, END08 )
     .    /
     .      'VXLIST',             '1', INT,  'TRUE',  'FALSE', 'FALSE' /
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG09, END09 )
     .    /
     .      'MASK',               '1', INT,  'TRUE',  'FALSE', 'FALSE',
     .      'MAXELE',             '1',  DP,  'TRUE',  'FALSE', 'FALSE' /
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG10, END10 )
     .    /
     .      'COMMENTS',           '1', CHR,  'TRUE',  'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG11, END11 )
     .    /
     .      'VCPNTR',             '1', INT,  'TRUE',  'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG12, END12 )
     .    /
     .      'NVC',                '1', INT,  'TRUE',  'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG13, END13 )
     .    /
     .      'VCLIST',             '1', INT,  'TRUE',  'FALSE', 'FALSE' /
 
 
      DATA ( CNAMES(I), SIZE(I), DTYPES(I), IDXFLG(I), NULLOK(I),
     .       FXDCNT(I),          I = BEG14, END14 )
     .    /
     .      'X_MIN',              '1',  DP, 'FALSE',  'FALSE', 'FALSE',
     .      'X_MAX',              '1',  DP, 'FALSE',  'FALSE', 'FALSE',
     .      'Y_MIN',              '1',  DP, 'FALSE',  'FALSE', 'FALSE',
     .      'Y_MAX',              '1',  DP, 'FALSE',  'FALSE', 'FALSE',
     .      'Z_MIN',              '1',  DP, 'FALSE',  'FALSE', 'FALSE',
     .      'Z_MAX',              '1',  DP, 'FALSE',  'FALSE', 'FALSE' /
 
 
 
C
C     Standard SPICE error handling.
C
      CALL CHKIN ( 'MKPLAT2' )
 
C
C     Display startup message.
C
      CALL TOSTDO ( ' ' )
      CALL TOSTDO ( '-----------------------------------------------' )
      CALL TOSTDO ( 'MKPLAT, Generate a plate model database kernel.' )
      CALL TOSTDO ( ' ' )
      CALL TOSTDO ( '              Version 2.0.2' )
      CALL TOSTDO ( '-----------------------------------------------' )
      CALL TOSTDO ( ' ' )
 
C
C     Get the command line.
C
      CALL GETCML ( LINE )
 
C
C     The first word in the command line, if it exists,
C     is the name of the command file.
C
      CALL NEXTWD ( LINE, CMDFIL, LINE )
 
C
C     If the command line is blank, prompt the user for the
C     name of the command input file.
C
      IF ( CMDFIL .EQ. ' ' ) THEN
         CALL TOSTDO ( ' ' )
         CALL TOSTDO ( 'Enter the name of the command file: ' )
         CALL PROMPT ( '> ', CMDFIL )
         CALL TOSTDO ( ' ' )
      END IF
 
C****************************************************************
C
C     Command File Keyword Definitions
C     --------------------------------
C
C     MODEL_DESCRIPTION, is a brief, descriptive summary of the model,
C                        and is a quoted character string.
C     Example:
C        MODEL_DESCRIPTION = 'NEAR/Eros-10000 plate model-Version 12'
C
C
C     CREATOR, is the person and/or group who created plate model,
C              and is a quoted character string.
C     Example:
C        CREATOR = 'Jim Miller, NAV/JPL'
C
C
C     BODY_NAME, is the name of the plate model body, and is a
C                quoted character string.
C     Example:
C        BODY_NAME = 'Eros'
C
C
C     NAIF_ID, is the NAIF ID of plate model body, and is an integer.
C              It must correspond to the BODY_NAME.
C     Example:
C        NAIF_ID = 2000433
C
C
C     VERSION, is an integer signifying the version number of the model.
C     Example:
C        VERSION = 12
C
C
C     VOXEL_SCALE, a double precision scale factor that multiplies
C                  the average plate extent to get the voxel size.
C     Example:
C        VOXEL_SCALE = 1.D0
C
C
C     IN_FILE, is the name of the input plate model flatfile, and is a
C              quoted character string.
C     Example:
C        IN_FILE = 'shape.dat'
C
C
C     OUT_FILE, is the name of output plate kernel file, and is a
C               quoted character string.
C     Example:
C        OUT_FILE = 'eros.plt'
C
C
C     ALBEDO_FILE, is the name of the albedo flatfile, and is a quoted
C                  character string.
C     Example:
C        ALBEDO_FILE = 'eros.alb'
C
C     NOTE: if the albedo file is 'NULL' (signifying no file), the
C           albedo of all plates will be set to a missing value
C           code of -1.D0 .
C
C
C     COMMENT_FILE, is the name of the comment textfile,
C                   and is a quoted character string.
C     Example:
C        COMMENT_FILE = 'extra_comments.txt'
C
C     NOTE: if the coment file is 'NULL' (signifying no file), the
C           extra comment field in the E-kernel will be set to null.
C
C
C     LEAPSECOND_FILE, is the name of leapsecond file, and is a
C                      quoted character string.
C     Example:
C        LEAPSECOND_FILE = 'naif0007.tls'
C
 
 
C
C     Load the command file into the kernel pool and attempt to get
C     the keyword values through pool retrieval routines.  Signal
C     error if any required variable is not found, or is found to
C     be invalid.
C
 
      CALL LDPOOL ( CMDFIL )
 
 
C
C     Retrieve the Model Description from the kernel pool.
C
      CALL GCPOOL ( CNAMES(1), 1, 1, N, MODESC, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', CNAMES(1)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve the creator's name and/or Affiliation.
C
      CALL GCPOOL ( CNAMES(3), 1, 1, N, CREATR, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', CNAMES(3)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve the Body Name.  Check to make sure it is actually a
C     real body.  Then convert it to upper case.
C
      CALL GCPOOL ( CNAMES(4), 1, 1, N, BDNAME, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', CNAMES(4)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve the NAIF ID Code of the body.  Check to make
C     it's real and matches the given body name.
C
      CALL GIPOOL ( CNAMES(5), 1, 1, N, NAIFID, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', CNAMES(5)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Check correctness of given body name and NAIF ID.
C     If one does not match the other, an error is signaled.
C     If neither is found, then we assume the given
C     body name and NAIF ID corresponds to a user defined object.
C
      CALL UCASE  ( BDNAME(1), BDNAME(1)     )
 
      CALL BODN2C ( BDNAME(1), BCODE, BODFND )
      CALL BODC2N ( NAIFID(1), BNAME, CODFND )
 
      IF ( BODFND .NEQV. CODFND ) THEN
         CALL SETMSG (   'Requested body # or '
     .                // 'NAIF ID # not found.            ' )
         CALL ERRCH  ( '#', BDNAME(1)                       )
         CALL ERRINT ( '#', NAIFID(1)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve Version Number.
C
      CALL GIPOOL ( CNAMES(6), 1, 1, N, VERSN, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', CNAMES(6)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve Voxel Scale Factor.
C
      CALL GDPOOL ( 'VOXEL_SCALE', 1, 1, N, VOXSCL, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', 'VOXEL_SCALE'                   )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve the name of the Plate Model Input Flatfile.
C
      CALL GCPOOL ( 'IN_FILE', 1, 1, N, INFILE, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword IN_FILE not found.  ')
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'                    )
      END IF
 
      IF ( .NOT. EXISTS( INFILE(1) ) ) THEN
         CALL SETMSG ( 'Input plate model file # not found ')
         CALL ERRCH  ( '#', INFILE(1)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve the name of the Plate Model Database Kernel Output File.
C
      CALL GCPOOL ( 'OUT_FILE', 1, 1, N, OUTFIL, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', 'OUT_FILE'                      )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
      IF ( EXISTS( OUTFIL(1) ) ) THEN
         CALL SETMSG ( 'Output plate model file # exists.  ')
         CALL ERRCH  ( '#', OUTFIL(1)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
 
C
C     Retrieve the name of the Albedo Input Flatfile.
C
      CALL GCPOOL ( 'ALBEDO_FILE', 1, 1, N, ALBFIL, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword ALBEDO_FILE not found.  ')
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'                        )
      END IF
 
      IF ( ALBFIL(1) .NE. 'NULL' ) THEN
         IF ( .NOT. EXISTS( ALBFIL(1) ) ) THEN
            CALL SETMSG ( 'Input albedo file # not found   ')
            CALL ERRCH  ( '#', ALBFIL(1)                    )
            CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'           )
         END IF
      END IF
 
C
C     Retrieve the name of the Comment Textfile.
C
      CALL GCPOOL ( 'COMMENT_FILE', 1, 1, N, COMFIL, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword COMMENT_FILE not found. ')
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'                        )
      END IF
 
      IF ( COMFIL(1) .NE. 'NULL' ) THEN
         IF ( .NOT. EXISTS( COMFIL(1) ) ) THEN
            CALL SETMSG ( 'Input comment file # not found  ')
            CALL ERRCH  ( '#', COMFIL(1)                    )
            CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'           )
         END IF
      END IF
 
C
C     Retrieve the name of the Leapsecond Kernel.
C
      CALL GCPOOL ( 'LEAPSECOND_FILE', 1, 1, N, LSFILE, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', 'LEAPSECOND_FILE'               )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
      IF ( .NOT. EXISTS( LSFILE(1) ) ) THEN
         CALL SETMSG ( 'Leapsecond file # not found '       )
         CALL ERRCH  ( '#', LSFILE(1)                       )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
C
C     Retrieve the horizon mask action flag.
C
      CALL GCPOOL ( 'HORIZON_MASK', 1, 1, N, HORFLG, FOUND )
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Command file keyword # not found.  ')
         CALL ERRCH  ( '#', 'HORIZON_MASK'                  )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
 
      IF ( (HORFLG(1) .NE. 'Y') .AND. (HORFLG(1) .NE. 'N')  ) THEN
         CALL SETMSG ( 'HORIZON_MASK not Y or N '           )
         CALL SIGERR ( 'SPICE(BADCOMMANDFILE)'              )
      END IF
 
 
 
C
C     All command veriables successfully loaded; list them
C     for the user.
C
      CALL TOSTDO ( '                          ' )
      CALL TOSTDO ( 'Summary of Input Variables' )
      CALL TOSTDO ( '--------------------------' )
      CALL TOSTDO ( '                          ' )
 
      CALL STDIO  ( 'STDOUT', STDOUT )
 
      CALL WRPOOL ( STDOUT )
 
 
C*******************************************************************
C     Load the Leapsecond kernel.
C*******************************************************************
 
      CALL LDPOOL ( LSFILE(1) )
 
 
C*******************************************************************
C     Read the comment file into a character array.
C*******************************************************************
 
      IF ( COMFIL(1) .NE. 'NULL' ) THEN
         NLINE = 0
         CALL RDTEXT ( COMFIL(1), INCOM, EOF )
         DO WHILE ( .NOT. EOF )
            NLINE = NLINE + 1
            IF( NLINE .GT. MAXCOM ) THEN
               CALL SETMSG ( 'Number of lines in # exceeds 1000' )
               CALL ERRCH  ( '#', COMFIL(1)                      )
               CALL SIGERR ( 'SPICE(BADCOMMENTFILE)'             )
            END IF
            COMMNT(NLINE) = INCOM
            CALL RDTEXT ( COMFIL(1), INCOM, EOF )
         END DO
      ELSE
         NLINE = 0
      END IF
 
C*******************************************************************
C     Read the plate and albedo flatfiles.
C*******************************************************************
 
      CALL TOSTDO ( 'Reading plate model flatfile...' )
 
      CALL RDFFPL ( INFILE(1), NV, VID, X,  Y,  Z,
     .                         NP, PID, V1, V2, V3 )
 
      CALL RDFFAL ( ALBFIL(1), NP, PID, ALBEDO )
 
 
C
C     For each plate, load into an array the coordinates
C     of its vertices.
C
 
      DO I = 1, NP(1)
 
         VERT(1,1,I) = X(V1(I))
         VERT(1,2,I) = X(V2(I))
         VERT(1,3,I) = X(V3(I))
 
         VERT(2,1,I) = Y(V1(I))
         VERT(2,2,I) = Y(V2(I))
         VERT(2,3,I) = Y(V3(I))
 
         VERT(3,1,I) = Z(V1(I))
         VERT(3,2,I) = Z(V2(I))
         VERT(3,3,I) = Z(V3(I))
 
      END DO
 
C*******************************************************************
C     Compute plates' unit normal, center, area and length of
C     longest side.
C*******************************************************************
 
      CALL TOSTDO ( 'Calculating plate normals, centers, etc...' )
 
      CALL PLCALC ( NP(1), VERT, NORM, CENT, AREA, LSIDE  )
 
C*******************************************************************
C     Given a vertex, which plates does it belong to?
C*******************************************************************
 
 
      CALL TOSTDO ( 'Determining vertex-plate membership...' )
 
      CALL VRTCOM ( NP(1), V1, V2, V3, VCPNTR, NVC, VCLIST )
 
 
C*******************************************************************
C     Generate the spatial index.
C*******************************************************************
 
 
      CALL TOSTDO ( 'Generating Spatial Index...' )
 
      CALL MKSPIN  ( NP(1),  VERT,
     .               VOXSCL, NVOX,   VOXSIZ, VOXORI,
     .               NVXTOT, VXPNTR, NVX,    VXLIST,
     .               EXTENT                           )
 
 
C*******************************************************************
C     Construct an horizon mask.
C*******************************************************************
 
 
      IF ( HORFLG(1) .EQ. 'Y' ) THEN
         CALL TOSTDO ( 'Constructing First Order Horizon Mask...' )
         CALL MKMASK ( NP(1), VERT, NORM, CENT,
     .                 V1,    V2,   V3,
     .                 PMASK, MAXELE            )
C         WRITE(*,*) 'after mkmask, np = ',   NP(1)
         DO I = 1, NP(1)
C            WRITE(*,*) 'plate ID, pmask, maxele = ', I,
C     .                  PMASK(I), MAXELE(I)
         END DO
      ELSE
         DO I = 1, NP(1)
            PMASK(I)  = 1
            MAXELE(I) = 0.D0
         END DO
      END IF
 
C*******************************************************************
C     Initialize plate kernel
C*******************************************************************
 
C
C     Create body+version suffix for table names.
C     Any blanks or hyphens in the body name should be replaced
C     by an underscore ('_') to satisfy the database kernel
C     naming rule.
C
      BVID = '_#_#'
 
      CALL REPMC  ( BVID, '#', BDNAME(1), BVID )
      CALL REPMI  ( BVID, '#', VERSN(1),  BVID )
 
      LB = LASTNB ( BVID )
      CALL REPLCH ( BVID(1:LB), ' ', '_', BVID(1:LB) )
      CALL REPLCH ( BVID(1:LB), '-', '_', BVID(1:LB) )
 
C
C     Fill up column declarations array.
C
      DO I = 1, NCOLS
 
         DECLAR = 'FIXED_COUNT = #, DATATYPE = #, INDEXED = #, '
     .     //     'NULLS_OK = #, SIZE = #'
 
         CALL REPMC ( DECLAR, '#', FXDCNT(I),     DECLAR )
         CALL REPMC ( DECLAR, '#', DT(DTYPES(I)), DECLAR )
         CALL REPMC ( DECLAR, '#', IDXFLG(I),     DECLAR )
         CALL REPMC ( DECLAR, '#', NULLOK(I),     DECLAR )
         CALL REPMC ( DECLAR, '#', SIZE(I),       DECLAR )
 
         CDECLS(I) = DECLAR
 
      END DO
 
 
 
C*******************************************************************
C     Open new E-kernel file to hold the plate database.
C*******************************************************************
 
 
      CALL EKOPN  ( OUTFIL(1), 'PLATE_MODEL', 10000, HANDLE )
 
      CALL TOSTDO ( ' ' )
      CALL TOSTDO ( 'Opening plate kernel for writing...' )
      CALL TOSTDO ( ' ' )
 
 
C*******************************************************************
C     Write Segment One (HEADER).
C*******************************************************************
 
      SEGNO  = 1
 
      NCOL   = 9
 
      IC     = BEG01
      NREC   = 1
 
      TABNAM = 'HEADER'  //  BVID
 
 
      CALL EKIFLD ( HANDLE,        TABNAM,      NCOL,  NREC,
     .              CNAMES(IC),    CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the Model Description.
C
      CALL EKACLC ( HANDLE,  SEGNO,   CNAMES(IC),  MODESC,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
C
C     Determine and load the Date Created.
C
      CALL CURTIM ( DATE )
      TIM(1) = DATE
 
      CALL EKACLC ( HANDLE,  SEGNO,   CNAMES(IC+1),  TIM,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL TOSTDO ( ' ' )
      CALL TOSTDO ( 'Kernel creation date = ' // DATE )
      CALL TOSTDO ( ' ' )
 
 
C
C     Load the Creator.
C
      CALL EKACLC ( HANDLE,  SEGNO,   CNAMES(IC+2),  CREATR,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the Body Name.
C
      CALL EKACLC ( HANDLE,  SEGNO,   CNAMES(IC+3),  BDNAME,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the NAIF ID.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+4),  NAIFID,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the Version Number.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+5),  VERSN,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the Number of Vertices.
C
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+6),  NV,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the Number of Plates.
C
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+7),  NP,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
C
C     Load number of Comments in Comment Segment.
C
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+8),  NLINE,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
 
C*******************************************************************
C     Write Segment Two (VERTEX).
C*******************************************************************
 
      SEGNO  = 2
 
      NCOL   = 4
 
      IC     = BEG02
      NREC   = NV(1)
 
      TABNAM = 'VERTEX'  //  BVID
 
      CALL EKIFLD ( HANDLE,        TABNAM,     NCOL,  NREC,
     .              CNAMES(IC),    CDECLS(IC), SEGNO, RCPTRS )
 
C
C     Load the Vertex ID Array.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),  VID,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
C
C     Load the Vertex Coordinate Arrays.
C
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+1),  X,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+2),  Y,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+3),  Z,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Three (PLATE).
C*******************************************************************
 
      SEGNO  = 3
 
      NCOL   = 4
 
      IC     = BEG03
      NREC   = NP(1)
 
      TABNAM = 'PLATE'  //  BVID
 
 
      CALL EKIFLD ( HANDLE,       TABNAM,      NCOL,  NREC,
     .              CNAMES(IC),   CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the Plate ID array.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),  PID,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
C
C     Load the Plate Vertex List arrays.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+1),    V1,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+2),  V2,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+3),  V3,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Four  (ALBEDO).
C*******************************************************************
 
      SEGNO  = 4
 
      NCOL   = 1
 
      IC     = BEG04
      NREC   = NP(1)
 
      TABNAM = 'ALBEDO'  //  BVID
 
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the Plate Albedo array.
C
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC),  ALBEDO,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Five  (PLATE_INFO).
C*******************************************************************
 
      SEGNO  = 5
 
      NCOL   = 8
 
      IC     = BEG05
      NREC   = NP(1)
 
      TABNAM = 'PLATE_INFO'  //  BVID
 
      DO I = 1, NP(1)
         NORMX(I) = NORM(1,I)
         NORMY(I) = NORM(2,I)
         NORMZ(I) = NORM(3,I)
         CENTX(I) = CENT(1,I)
         CENTY(I) = CENT(2,I)
         CENTZ(I) = CENT(3,I)
      END DO
 
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the Plate Normal arrays.
C
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC),    NORMX,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+1),  NORMY,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+2),  NORMZ,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the Plate Center arrays.
C
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+3),  CENTX,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+4),  CENTY,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+5),  CENTZ,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the Plate Area array.
C
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+6),  AREA,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the Plate Longest Side array.
C
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+7),  LSIDE,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Six  (VOXEL_SUMMARY).
C*******************************************************************
 
      SEGNO  = 6
 
      NCOL   = 9
 
      IC     = BEG06
      NREC   = 1
 
      TABNAM = 'VOXEL_SUMMARY'  //  BVID
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
 
C
C     Load the voxel grid dimensions in voxel units.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),    NVOX(1),
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+1),  NVOX(2),
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+2),  NVOX(3),
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the voxel size in model units.
C
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+3),  VOXSIZ,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
C
C     Load the voxel grid origin in model units.
C
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+4),  VOXORI(1),
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX    )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+5),  VOXORI(2),
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX    )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+6),  VOXORI(3),
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX    )
 
C
C     Load the length of the pointer array to the voxel-plate
C     occupation list.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+7),  NVXTOT,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
C
C     Load the length of the voxel-plate occupation list.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC+8),  NVX,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Seven  (VOXEL_POINTERS).
C*******************************************************************
 
      SEGNO  = 7
 
      NCOL   = 1
 
      IC     = BEG07
      NREC   = NVXTOT
 
      TABNAM = 'VOXEL_POINTERS'  //  BVID
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the pointer array into the voxel occupancy list.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),  VXPNTR,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Eight  (VOXEL_LIST).
C*******************************************************************
 
      SEGNO  = 8
 
      NCOL   = 1
 
      IC     = BEG08
      NREC   = NVX
 
      TABNAM = 'VOXEL_LIST'  //  BVID
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the voxel occupancy list.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),  VXLIST,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Nine  (MASK).
C*******************************************************************
 
      SEGNO  = 9
 
      NCOL   = 2
 
      IC     = BEG09
      NREC   = NP(1)
 
      TABNAM = 'MASK'  //  BVID
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the horizon mask flag array.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),    PMASK,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+1),  MAXELE,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Ten (COMMENTS).
C*******************************************************************
 
 
      SEGNO  = 10
 
      NCOL   = 1
 
      IC     = BEG10
      NREC   = NLINE
 
      TABNAM = 'COMMENTS'  //  BVID
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the comment lines.
C
      CALL EKACLC ( HANDLE,  SEGNO,   CNAMES(IC),  COMMNT,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Eleven (VERTEX_POINTER).
C*******************************************************************
 
 
      SEGNO  = 11
 
 
      NCOL   = 1
 
      IC     = BEG11
      NREC   = NV(1)
 
      TABNAM = 'VERTEX_POINTER'  //  BVID
 
 
      CALL EKIFLD ( HANDLE,        TABNAM,      NCOL,  NREC,
     .              CNAMES(IC),    CDECLS(IC),  SEGNO, RCPTRS )
 
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),  VCPNTR,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
 
C*******************************************************************
C     Write Segment Twelve (VERTEX_SUMMARY).
C*******************************************************************
 
      SEGNO  = 12
 
      NCOL   = 1
 
      IC     = BEG12
      NREC   = 1
 
      TABNAM = 'VERTEX_SUMMARY'  //  BVID
 
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
C
C     Load the length of the vertex-plate correspondence list.
C
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),  NVC,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
 
C*******************************************************************
C     Write Segment Thirteen (Vertex Ownership List).
C*******************************************************************
 
 
      SEGNO  = 13
 
      NCOL   = 1
 
      IC     = BEG13
      NREC   = NVC
 
      TABNAM = 'VERTEX_COMMON'  //  BVID
 
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
 
      CALL EKACLI ( HANDLE,  SEGNO,   CNAMES(IC),  VCLIST,
     .              ENTSZS,  NLFLGS,  RCPTRS,      WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Write Segment Fourteen (MODEL_EXTENT).
C*******************************************************************
 
      SEGNO  = 14
 
      NCOL   = 6
 
      IC     = BEG14
      NREC   = 1
 
      XMIN = EXTENT(1)
      XMAX = EXTENT(2)
      YMIN = EXTENT(3)
      YMAX = EXTENT(4)
      ZMIN = EXTENT(5)
      ZMAX = EXTENT(6)
 
 
      TABNAM = 'MODEL_EXTENT'  //  BVID
 
 
      CALL EKIFLD ( HANDLE,     TABNAM,      NCOL,  NREC,
     .              CNAMES(IC), CDECLS(IC),  SEGNO, RCPTRS )
 
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC),    XMIN,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+1),  XMAX,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+2),  YMIN,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+3),  YMAX,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+4),  ZMIN,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
      CALL EKACLD ( HANDLE,  SEGNO,   CNAMES(IC+5),  ZMAX,
     .              ENTSZS,  NLFLGS,  RCPTRS,        WKINDX  )
 
 
      CALL EKFFLD ( HANDLE,  SEGNO,   RCPTRS )
 
 
C*******************************************************************
C     Close Plate Kernel file.
C*******************************************************************
 
      CALL EKCLS ( HANDLE )
 
      CALL TOSTDO (' ')
      CALL TOSTDO ('Plate kernel created. Exiting program.')
      CALL TOSTDO (' ')
 
      STOP
      END
